package com.esignlive.app;

import android.annotation.TargetApi;
import android.content.Context;
import android.os.Build;
import android.support.annotation.Nullable;
import android.util.AttributeSet;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

/**
 * Created by smoetakef on 2016-11-22.
 */

public class BlockingMessageView extends LinearLayout {

    // Constants
    private final long ANIMATION_DURATION = 300;

    // properties - UI components
    private LinearLayout backgroundView;
    private ProgressBar progressBar;
    private TextView textView;

    // Constructors
    public BlockingMessageView(Context context) {
        super(context);
        this.initialize(context);
    }
    public BlockingMessageView(Context context, AttributeSet attrs) {
        super(context, attrs);
        this.initialize(context);
    }
    public BlockingMessageView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.initialize(context);
    }
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public BlockingMessageView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        this.initialize(context);
    }

    // Setup
    private void initialize(Context context) {
        this.getReferences(context);
        this.setupUI();
    }
    private void getReferences(Context context) {
        View rootView = inflate(context, R.layout.view_spinning_wheel, this);
        this.backgroundView = (LinearLayout) rootView.findViewById(R.id.spinningWheelView_background);
        this.progressBar = (ProgressBar) rootView.findViewById(R.id.spinningWheelView_progressBar);
        this.textView = (TextView) rootView.findViewById(R.id.spinningWheelView_progressTextView);
    }
    private void setupUI() {
        this.hide(false, null);
    }

    // Methods
    public void show(@Nullable String message, boolean animated, @Nullable final Runnable completion) {
        this.backgroundView.setVisibility(VISIBLE);
        final String txt = message == null ? "" : message;
        this.textView.setText(txt);
        Runnable endAction = new Runnable() {
            @Override
            public void run() {
                BlockingMessageView.this.backgroundView.setAlpha(1);
                if (completion != null) {
                    completion.run();
                }
            }
        };
        if (animated) {
            this.backgroundView.animate().setDuration(this.ANIMATION_DURATION).alpha(1).withEndAction(endAction);
        } else {
            endAction.run();
        }
    }
    public void hide(boolean animated, @Nullable final Runnable completion) {
        Runnable endAction = new Runnable() {
            @Override
            public void run() {
                BlockingMessageView.this.backgroundView.setAlpha(0);
                BlockingMessageView.this.backgroundView.setVisibility(INVISIBLE);
                if (completion != null) {
                    completion.run();
                }
            }
        };
        if (animated) {
            this.backgroundView.animate().setDuration(this.ANIMATION_DURATION).alpha(0).withEndAction(endAction);
        } else {
            endAction.run();
        }
    }
}
