package com.esignlive.app;

import android.app.Activity;
import android.content.Intent;
import android.content.res.AssetManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.ActionMode;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.esignlive.sdk.AuthenticationListener;
import com.esignlive.sdk.ESignLive;
import com.esignlive.sdk.ESignLiveRootActivityProvider;
import com.esignlive.sdk.datamodel.User;
import com.esignlive.sdk.UserInfoListener;
import com.esignlive.sdk.datamodel.manage.EmailMissingException;
import com.esignlive.sdk.datamodel.manage.EmailValidationException;
import com.esignlive.sdk.datamodel.manage.FirstNameMissingException;
import com.esignlive.sdk.datamodel.manage.LastNameMissingException;
import com.esignlive.sdk.datamodel.manage.Role;
import com.esignlive.sdk.datamodel.manage.RoleAlreadyAddedException;
import com.esignlive.sdk.datamodel.manage.Transaction;
import com.esignlive.sdk.datamodel.manage.TransactionNotFoundException;
import com.esignlive.sdk.preparation.Approval;
import com.esignlive.sdk.preparation.AutofillField;
import com.esignlive.sdk.preparation.CreateTransactionViewModel;
import com.esignlive.sdk.preparation.FormField;
import com.esignlive.sdk.preparation.Signature;
import com.esignlive.sdk.preparation.SignatureMissingException;
import com.esignlive.sdk.preparation.TextField;
import com.esignlive.sdk.utility.Error;
import com.esignlive.sdk.utility.Util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;


public class HomeActivity extends Activity implements View.OnClickListener,CreateTransactionViewModel.CreateTransactionListener, ESignLiveRootActivityProvider {

    // Properties
    private Button logoutButton;
    private Button createTransactionButton;
    private Button listOfTransactionsButton;
    private Button reloadUserData;
    private Button createTransactionLocallyButton;
    private TextView serverAddressField, sdkVersionTextView;
    private BlockingMessageView blockingMessageView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_home);
        this.getReferences();
        this.setupListeners();
        ESignLive.getInstance().setRootActivityProvider(this);

        boolean shouldPerformLogin = this.getIntent().getBooleanExtra("performLogin",false);
        if (shouldPerformLogin){
            this.performAutoLogin();
        }
        this.sdkVersionTextView.setText("SDK Version: " + ESignLive.getInstance().getVersion());
        this.serverAddressField.setText(ESignLive.getInstance().getServerAddress());
        this.blockingMessageView.hide(false, null);
    }

    // UI components
    private void getReferences() {
        this.logoutButton = (Button) this.findViewById(R.id.logoutButton);
        this.createTransactionButton  = (Button) this.findViewById(R.id.createTransactionButton);
        this.listOfTransactionsButton = (Button) this.findViewById(R.id.transactionListButton);
        this.reloadUserData           = (Button) this.findViewById(R.id.reloadUserDataButton);
        this.createTransactionLocallyButton = (Button) this.findViewById(R.id.createTransactionLocallyButton);
        this.serverAddressField       = (TextView) this.findViewById(R.id.textView2);
        this.sdkVersionTextView       = (TextView) this.findViewById(R.id.SDK_version_textView);
        this.blockingMessageView      = (BlockingMessageView) this.findViewById(R.id.homeActivity_blockingMessageView);
    }
    private void setupListeners() {
        this.logoutButton.setOnClickListener(this);
        this.createTransactionButton.setOnClickListener(this);
        this.listOfTransactionsButton.setOnClickListener(this);
        this.reloadUserData.setOnClickListener(this);
        this.createTransactionLocallyButton.setOnClickListener(this);
    }

    // Implement interfaces
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.logoutButton:
                UserData.setUserNotLoggedIn();
                ESignLive.getInstance().logout();
                this.navigateToLandingPageActivity();
                break;

            case  R.id.transactionListButton:
                this.navigateToTransactionListActivity();
                break;

            case  R.id.createTransactionButton:
                ESignLive.getInstance().createNewTransactionUsingSdkUserInterface();
                break;

            case R.id.reloadUserDataButton:
                ESignLive.getInstance().reloadUserData(new UserInfoListener() {
                    @Override
                    public void userInfoDownloaded(User user) {
                        Log.d("APP", "Local listener - user Data Reloading Finished with success:\n" + user.toString());
                        Toast.makeText(HomeActivity.this, "UserData Loaded:\n-----------------\n" + user.toString(),Toast.LENGTH_LONG).show();
                    }

                    @Override
                    public void userInfoDownloadingFailed(Error error) {
                        Log.d("APP", "Local listener - user Data Reloading Finished with error:\n" + error.toString());
                        Toast.makeText(HomeActivity.this,"Failed to reload User Data.", Toast.LENGTH_SHORT).show();
                    }
                });
                break;
            case R.id.createTransactionLocallyButton:
                createLocalTransaction();
            default:
                break;
        }
    }

    private void createLocalTransaction() {


        try {
            Transaction transaction = CreateTransactionViewModel.createTransaction("Locally created transaction");

            CreateTransactionViewModel.setInPersonSigning(transaction.getId(),true);

            Role role = CreateTransactionViewModel.addRole(transaction.getId(),"Test","Signer","test@example.com","ACME",null);
            Approval approval = new Approval(null,role.getId());
            FormField signature = new Signature(null, Signature.Type.CAPTURE_SIGNATURE, 0, 320, 350, 200,50);
            approval.addField(signature);
            List<Approval> approvalsList = new ArrayList();
            approvalsList.add(approval);

            Uri docUri = moveAssetToHomeFolderIfNeeded("ScrumGuideJuly2013.pdf");
            if (docUri == null) {
                Log.e("APP","Doc not found");
                return;
            }


            CreateTransactionViewModel.addDocument(transaction.getId(), docUri,"123","ScrumGuideJuly2013","", approvalsList, new CreateTransactionViewModel.AddDocumentListener() {
                @Override
                public void documentAddSuccessful(final String transactionId, String documentId) {
                    Util.runOnMainThread(new Util.MainThreadListener() {
                        @Override
                        public void executeOnMainThread() {
                            CreateTransactionViewModel.sendTransaction(transactionId);
                            ESignLive.getInstance().startTransactionWithId(transactionId,false);
                        }
                    });
                }

                @Override
                public void documentAddFailed(String transactionId, String documentId, Error error) {
                    Log.e("APP",error.toString());
                }
            });


        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // eSignLive Logic
    private void performAutoLogin() {
        UserData userData = UserData.lastLoggedInUserData();
        String lastEnteredEmail, lastEnteredPassword;
        try {
            lastEnteredEmail = userData.getEmail();
            lastEnteredPassword = userData.getPassword();
        } catch (Exception e) {
            lastEnteredEmail = null;
            lastEnteredPassword = null;
        }
        this.blockingMessageView.show("Login in progress",true, null);
        ESignLive.getInstance().login(lastEnteredEmail, lastEnteredPassword, new AuthenticationListener() {
            @Override
            public void loginSucceeded(User user, boolean isOnlineLogin) {
                HomeActivity.this.blockingMessageView.hide(true, null);
                Log.d("APP","logged in");
            }

            @Override
            public void loginFailed(Error error) {
                HomeActivity.this.blockingMessageView.hide(true, null);
                Log.d("APP","login error"+error.toString());
                UserData.setUserNotLoggedIn();
                navigateToLandingPageActivity();
            }
        });
    }

    // Navigation
    private void navigateToLandingPageActivity() {
        Intent intent = new Intent(HomeActivity.this,LandingPageActivity.class);
        HomeActivity.this.finishActivity(0);
        startActivity(intent);
    }
    private void navigateToTransactionListActivity() {
        Intent intent = new Intent(this, TransactionListActivity.class);
        this.startActivity(intent);
    }

    // Activity LifeCycle
    @Override
    public void onActionModeFinished(ActionMode mode) {
        super.onActionModeFinished(mode);
        this.blockingMessageView.hide(true, null);
    }
    @Override
    public void onBackPressed() {
        super.onBackPressed();
        this.blockingMessageView.hide(true, null);
    }

    private Uri moveAssetToHomeFolderIfNeeded(String filename) {
        AssetManager assetManager = getAssets();
        InputStream in = null;
        OutputStream out = null;
        try
        {
            in = assetManager.open(filename);
            String filePath = getFilesDir().toString() +"/" + filename;
            File pdf = new File(filePath);
            if (pdf.exists()) {
                return Uri.fromFile(pdf);
            }
            out = new FileOutputStream(filePath);
            copyFile(in, out);
            in.close();
            out.flush();
            out.close();
            return Uri.fromFile(new File(filePath));
        }
        catch(Exception e)
        {
            Log.e("tag", e.getMessage());
        }
        return null;

    }

    private void copyFile(InputStream in, OutputStream out) throws IOException {
        byte[] buffer = new byte[1024];
        int read;
        while((read = in.read(buffer)) != -1){
            out.write(buffer, 0, read);
        }
    }

    @Override
    public void createTransactionSucceeded(String s) {

    }

    @Override
    public void createTransactionFromTemplateSucceeded(String s, String s1) {

    }

    @Override
    public void createTransactionFailed(Error error) {
        Log.e("APP","createTransactionFailed " + error);
    }

    @Override
    public Activity getRootActivityForSdkUserInterface() {
        return this;
    }
}
