package com.esignlive.app;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.TextInputEditText;
import android.util.Log;
import android.view.ActionMode;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.esignlive.sdk.ESignLive;
import com.esignlive.sdk.SignUpListener;
import com.esignlive.sdk.datamodel.User;
import com.esignlive.sdk.utility.Error;

public class SignupActivity extends Activity implements View.OnClickListener {

    // Properties - UI
    private Button registerButton;
    private TextInputEditText firstNameTextView, lastNameTextView, emailTextView;
    private TextInputEditText passwordEditText;
    private BlockingMessageView blockingMessageView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_signup);
        this.getReferences();
        this.setListeners();
        this.blockingMessageView.hide(false, null);
    }

    // UI components
    private void getReferences() {
        this.registerButton    = (Button) findViewById(R.id.email_register_button);
        this.firstNameTextView = (TextInputEditText) findViewById(R.id.first_name);
        this.lastNameTextView  = (TextInputEditText) findViewById(R.id.last_name);
        this.emailTextView     = (TextInputEditText) findViewById(R.id.email);
        this.passwordEditText  = (TextInputEditText) findViewById(R.id.password);
        this.blockingMessageView = (BlockingMessageView) findViewById(R.id.signup_blockingMessageView);
    }
    private void setListeners() {
        this.registerButton.setOnClickListener(this);
    }

    // Navigation
    public static void navigateToHomeActivity(Context context, boolean performLogin){
        Intent intent = new Intent(context, HomeActivity.class);
        intent.putExtra("performLogin",performLogin);
        context.startActivity(intent);
    }

    // Implementing interfaces
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.email_register_button:
                this.blockingMessageView.show("Signup in progress",true, null);
                ESignLive.getInstance().signup(this.firstNameTextView.getText().toString(), this.lastNameTextView.getText().
                        toString(), this.emailTextView.getText().toString(), this.passwordEditText.getText().toString(), new SignUpListener() {
                    @Override
                    public void signUpSucceeded(User user) {
                        SignupActivity.this.blockingMessageView.hide(true, null);
                        UserData.storeLoggedInUserDetails(SignupActivity.this.emailTextView.getText().toString(),SignupActivity.this.passwordEditText.getText().toString());
                        SignupActivity.this.navigateToHomeActivity(SignupActivity.this,false);
                    }

                    @Override
                    public void signUpFailed(Error error) {
                        SignupActivity.this.blockingMessageView.hide(true, null);
                        Log.d("signup", "signup failed with error" + error.toString());
                        Toast.makeText(SignupActivity.this, error.getMessage(), Toast.LENGTH_LONG).show();
                    }
                });
                break;
            default:
                break;
        }
    }

    // Activity LifeCycle
    @Override
    public void onActionModeFinished(ActionMode mode) {
        super.onActionModeFinished(mode);
        SignupActivity.this.blockingMessageView.hide(true, null);
    }
    @Override
    public void onBackPressed() {
        super.onBackPressed();
        SignupActivity.this.blockingMessageView.hide(true, null);
    }
}
