package com.esignlive.app;

import android.app.Activity;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.util.Log;
import android.view.ActionMode;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Toast;

import com.esignlive.sdk.ESignLive;
import com.esignlive.sdk.datamodel.manage.Transaction;
import com.esignlive.sdk.TransactionListListener;
import com.esignlive.sdk.utility.Error;

import java.util.ArrayList;


public class TransactionListActivity extends Activity implements AdapterView.OnItemClickListener{

    private ListView transactionListView;
    private BlockingMessageView blockingMessageView;
    private Spinner transactionFilterSpinner;
    Transaction.TransactionFilter currentTransactionType = Transaction.TransactionFilter.INBOX;
    Transaction.TransactionFilter [] types = {
            Transaction.TransactionFilter.INBOX,
            Transaction.TransactionFilter.DRAFTS,
            Transaction.TransactionFilter.CREATED_BY_CURRENT_USER,
            Transaction.TransactionFilter.REQUIRES_YOUR_SIGNATURE,
            Transaction.TransactionFilter.IN_PROGRESS,
            Transaction.TransactionFilter.EXPIRING_SOON,
            Transaction.TransactionFilter.COMPLETED,
            Transaction.TransactionFilter.TRASHED,
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_transaction_list);
        this.getReferences();
        this.setupTransactionFieldSpinner();
        this.setupListeners();
        this.blockingMessageView.hide(false, null);
    }

    private void getReferences() {
        this.transactionListView      = (ListView) findViewById(R.id.transactionListView);
        this.transactionFilterSpinner = (Spinner) findViewById(R.id.transactionFilterSpinner);
        this.blockingMessageView = (BlockingMessageView) findViewById(R.id.transactionListActivity_blockingMessageView);
    }

    private void setupTransactionFieldSpinner() {
        ArrayList<String> filterDescriptions = new ArrayList<String>();
        for (Transaction.TransactionFilter type : this.types) {
            filterDescriptions.add(type.toString());
        }
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, filterDescriptions);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        this.transactionFilterSpinner.setAdapter(adapter);
        this.transactionFilterSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                TransactionListActivity.this.currentTransactionType = TransactionListActivity.this.types[position];
                TransactionListActivity.this.updateTransactionList();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void updateTransactionList() {
        Log.d("APP", "showing transaction list for filter :" + this.currentTransactionType.toString());
        this.blockingMessageView.show("Loading Transactions", true, null);
        ESignLive.getInstance().getTransactionList(this.currentTransactionType, 0, 20, new TransactionListListener() {
            @Override
            public void downloadTransactionListSucceeded(Transaction.TransactionFilter transactionFilter, int startIndex, ArrayList<Transaction> transactions, int transactionsCount) {
                TransactionListActivity.this.blockingMessageView.hide(true, null);
                Log.d("APP", "Local listener - Transaction list fetch successful:" + transactions.size() + " transaction(s) fetched.");
                for (Transaction transaction : transactions) {
                    Log.d("APP", transaction.toString());
                }
                Toast.makeText(TransactionListActivity.this, "Transactions loaded with success", Toast.LENGTH_SHORT).show();
                TransactionListAdapter adapter = new TransactionListAdapter(TransactionListActivity.this, transactions);
                TransactionListActivity.this.transactionListView.setAdapter(adapter);
            }

            @Override
            public void downloadTransactionListFailed(Error error) {
                TransactionListActivity.this.blockingMessageView.hide(true, null);
                Log.d("APP", "Local listener - Transaction list fetch failed with error: " + error.toString());
                Toast.makeText(TransactionListActivity.this,"Loading transaction list for type (" + TransactionListActivity.this.currentTransactionType.toString() + ") failed.",Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void setupListeners()
    {
        this.transactionListView.setOnItemClickListener(this);
    }

    // Activity LifeCycle
    @Override
    public void onActionModeFinished(ActionMode mode) {
        super.onActionModeFinished(mode);
        this.blockingMessageView.hide(true, null);
    }
    @Override
    public void onBackPressed() {
        super.onBackPressed();
        this.blockingMessageView.hide(true, null);
    }


    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        Transaction transaction = (Transaction) this.transactionListView.getAdapter().getItem(position);
//        this.blockingMessageView.show("Loading ...", true, null);
        ESignLive.getInstance().startTransactionWithId(transaction.getId(), true);
    }
}
